unit frmMainU;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, hyieutils, iexBitmaps, hyiedefs,
  iesettings, iexRulers, Vcl.Menus, Vcl.ComCtrls, Vcl.StdCtrls, Vcl.ExtCtrls,
  ieopensavedlg, System.Contnrs, imageenview, ieview, ievect, Math, spHostDll;

type
  TfrmMain = class(TForm)
    PaintView: TImageEnVect;
    openDialog: TOpenImageEnDialog;
    Panel1: TPanel;
    listBoxCategory: TListBox;
    listBoxNames: TListBox;
    Panel4: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Panel3: TPanel;
    btnLoadPlugins: TButton;
    btnAbout: TButton;
    btnExecute: TButton;
    progressPlug: TProgressBar;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    itemOpen: TMenuItem;
    itemSave: TMenuItem;
    chkSubDir: TCheckBox;
    saveDialog: TSaveImageEnDialog;
    Panel2: TPanel;
    btnUndo: TButton;
    //procedure OnProgressPlug(done: Integer; total: Integer);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure itemOpenClick(Sender: TObject);
    procedure itemSaveClick(Sender: TObject);
    procedure btnLoadPluginsClick(Sender: TObject);
    procedure listBoxCategoryClick(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure btnExecuteClick(Sender: TObject);
    procedure btnUndoClick(Sender: TObject);
  private
    { Private declarations }
    _undoMap: TIEBitmap;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}
//---------------------------------------------------------------------------
procedure onProgressPlug(done: Integer; total: Integer) stdcall;
begin
if (total > 0) and (total >= done) then
   frmMain.progressPlug.Position := Ceil((done * 1.0 / total) * 100)
else
   frmMain.progressPlug.Position := 0;
end;
//---------------------------------------------------------------------------
procedure TfrmMain.FormCreate(Sender: TObject);
begin
IEGlobalSettings.AutoFragmentBitmap := False;
end;
//---------------------------------------------------------------------------
procedure TfrmMain.FormShow(Sender: TObject);
var
  lRet: Boolean;
begin
lRet := sphInit(HWND(self.Handle), onProgressPlug);
PaintView.SelectionMaskDepth  := 8;
_undoMap := TIEBitmap.Create(PaintView.IEBitmap);
end;
//---------------------------------------------------------------------------
procedure TfrmMain.itemOpenClick(Sender: TObject);
begin
if (openDialog.Execute) then
   begin
   PaintView.io.LoadFromFile(openDialog.FileName);
   if PaintView.IEBitmap.PixelFormat <> ie24RGB then
   	  PaintView.Proc.ConvertTo24Bit();
   _undoMap.Free;
   _undoMap := TIEBitmap.Create(PaintView.IEBitmap);
   end;
end;
//---------------------------------------------------------------------------
procedure TfrmMain.itemSaveClick(Sender: TObject);
begin
if (saveDialog.Execute) then
   PaintView.io.SaveToFile(saveDialog.FileName);
end;
//---------------------------------------------------------------------------
procedure TfrmMain.btnLoadPluginsClick(Sender: TObject);
var
   filePath: UnicodeString;
   catName: PWideChar;
begin
listBoxCategory.Clear();
listBoxNames.Clear();
filepath :=  ExtractFilePath(Application.ExeName) + '\8bfsamples\';
sphEnumerateDirectory(PWideChar(filepath), chkSubDir.Checked);
while (True) do
  begin
  catName := sphGetCategory();
  if (catName = nil) then
	   break;
  listBoxCategory.Items.Add(UnicodeString(catName));
  end;
if (listBoxCategory.Count > 0) then
   begin
   listBoxCategory.ItemIndex := 0;
   listBoxCategoryClick(self);
   end;
end;
//---------------------------------------------------------------------------
procedure TfrmMain.listBoxCategoryClick(Sender: TObject);
var
   entryName: PWideChar;
begin
listBoxNames.Clear();
sphSelectCategory(PWideChar(listBoxCategory.Items.Strings[listBoxCategory.ItemIndex]));
while (True) do
  begin
  entryName := sphGetEntry();
  if (entryName = nil) then
	   break;
  listBoxNames.Items.Add(UnicodeString(entryName));
  end;
if (listBoxNames.Count > 0) then
   listBoxNames.ItemIndex := 0;
end;
//---------------------------------------------------------------------------
procedure TfrmMain.btnAboutClick(Sender: TObject);
var
   FP: Pointer;
   nRet: Integer;
begin
if (listBoxCategory.Count = 0) or (listBoxNames.Count = 0) then
   exit;
FP := DisableTaskWindows(HWND(self.Handle));
nRet := sphPluginAbout(PWideChar(listBoxCategory.Items.Strings[listBoxCategory.ItemIndex]),
               PWideChar(listBoxNames.Items.Strings[listBoxNames.ItemIndex]));
EnableTaskWindows(FP);
Panel2.Caption := 'Ret code: ' + IntToStr(nRet);
end;
//---------------------------------------------------------------------------
procedure TfrmMain.btnExecuteClick(Sender: TObject);
var
  h, w, nRet: Integer;
  FP, maskScanline: Pointer;
  boundRect: TRect;
  pBR : PRect;
begin
if (listBoxCategory.Count = 0) or (listBoxNames.Count = 0) then
   exit;
h := PaintView.IEBitmap.Height;
w := PaintView.IEBitmap.Width;
FP := DisableTaskWindows(HWND(self.Handle));
maskScanline := nil;
pBR := nil;
if (PaintView.Selected) then
   begin
   maskScanline := PaintView.SelectionMask.ScanLine[h-1];
   boundRect.left := PaintView.SelectionMask.X1;
   boundRect.right := PaintView.SelectionMask.X2;
   boundRect.top := PaintView.SelectionMask.Y1;
   boundRect.bottom := PaintView.SelectionMask.Y2;
   pBR := @boundRect;
   end;
sphSetImage(PaintView.IEBitmap.Scanline[h-1], maskScanline, w, h, 4);
nRet := sphPluginExecute(PWideChar(listBoxCategory.Items.Strings[listBoxCategory.ItemIndex]),
		                     PWideChar(listBoxNames.Items.Strings[listBoxNames.ItemIndex]),
                         pBR);
EnableTaskWindows(FP);
PaintView.Update();
progressPlug.Position := 0;
progressPlug.Update();
Panel2.Caption := 'Ret code: ' + IntToStr(nRet);
end;
//---------------------------------------------------------------------------
procedure TfrmMain.btnUndoClick(Sender: TObject);
begin
  _undoMap.DrawToTIEBitmap(PaintView.IEBitmap, 0, 0);
  PaintView.Update;
end;
//---------------------------------------------------------------------------
end.
